package com.tenlionsoft.aimz_k.adapter

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.databinding.ViewDataBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgMyAudioBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgMyBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgMyFileBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgMyImgBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgMyVideoBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgOtherAudioBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgOtherBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgOtherFileBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgOtherImgBinding
import com.tenlionsoft.aimz_k.databinding.ItemMsgOtherVideoBinding
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgTypeStateEnum
import com.tenlionsoft.aimz_k.viewmodel.ChatPageViewModel
import com.tenlionsoft.baselib.base.BaseBindingAdapter

class ChatMsgAdapter(datas: List<MsgBean>, val viewModel: ChatPageViewModel) :
    BaseBindingAdapter<MsgBean, ViewDataBinding>(datas) {

    override fun getItemBinding(parent: ViewGroup, viewType: Int): ViewDataBinding {
        when (viewType) {
            //发送文本
            MsgTypeStateEnum.MSG_TO_OTHER_TXT ->
                return ItemMsgOtherBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )

            //收到文本信息
            MsgTypeStateEnum.MSG_FROM_OTHER_TXT ->
                return ItemMsgMyBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )

            //发送视频
            MsgTypeStateEnum.MSG_TO_OTHER_MOVIE ->
                return ItemMsgOtherVideoBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )

            //收到视频
            MsgTypeStateEnum.MSG_FROM_OTHER_MOVIE ->
                return ItemMsgMyVideoBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )

            //发送图片
            MsgTypeStateEnum.MSG_TO_OTHER_IMG ->
                return ItemMsgOtherImgBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )

            //收到图片
            MsgTypeStateEnum.MSG_FROM_OTHER_IMG ->
                return ItemMsgMyImgBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )

            //语音
            MsgTypeStateEnum.MSG_TO_OTHER_VOICE ->
                return ItemMsgMyAudioBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )
            //语音
            MsgTypeStateEnum.MSG_FROM_OTHER_VOICE ->
                return ItemMsgOtherAudioBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )
            //发送文件
            MsgTypeStateEnum.MSG_TO_OTHER_FILE ->
                return ItemMsgOtherFileBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )

            //收到文件
            MsgTypeStateEnum.MSG_FROM_OTHER_FILE -> {
                return ItemMsgMyFileBinding.inflate(
                    LayoutInflater.from(parent.context),
                    parent,
                    false
                )
            }

            else -> {
                throw IllegalArgumentException("Invalid view type")
            }
        }
    }

    override fun getItemViewType(position: Int): Int {
        return list[position].msgType!!
    }

    override fun bindItem(holder: BaseViewHolder<ViewDataBinding>, position: Int) {
        val itemType = getItemViewType(position)
        holder.binding.root.setOnClickListener {
            viewModel.onItemClickListener?.onItemClick(list[position])
        }
        when (itemType) {
            //发送文本
            MsgTypeStateEnum.MSG_TO_OTHER_TXT -> {
                (holder.binding as ItemMsgOtherBinding).pos = position
                (holder.binding as ItemMsgOtherBinding).bean = list[position]
                (holder.binding as ItemMsgOtherBinding).state = list[position].status
            }
            //收到文本信息
            MsgTypeStateEnum.MSG_FROM_OTHER_TXT -> {
                (holder.binding as ItemMsgMyBinding).pos = position
                (holder.binding as ItemMsgMyBinding).bean = list[position]
                (holder.binding as ItemMsgMyBinding).state = list[position].status
            }
            //发送视频
            MsgTypeStateEnum.MSG_TO_OTHER_MOVIE -> {
                (holder.binding as ItemMsgOtherVideoBinding).pos = position
                (holder.binding as ItemMsgOtherVideoBinding).bean = list[position]
                (holder.binding as ItemMsgOtherVideoBinding).state = list[position].status
            }
            //收到视频
            MsgTypeStateEnum.MSG_FROM_OTHER_MOVIE -> {
                (holder.binding as ItemMsgMyVideoBinding).pos = position
                (holder.binding as ItemMsgMyVideoBinding).bean = list[position]
                (holder.binding as ItemMsgMyVideoBinding).state = list[position].status
            }
            //发送图片
            MsgTypeStateEnum.MSG_TO_OTHER_IMG -> {
                (holder.binding as ItemMsgOtherImgBinding).pos = position
                (holder.binding as ItemMsgOtherImgBinding).bean = list[position]
                (holder.binding as ItemMsgOtherImgBinding).state = list[position].status
            }
            //收到图片
            MsgTypeStateEnum.MSG_FROM_OTHER_IMG -> {
                (holder.binding as ItemMsgMyImgBinding).pos = position
                (holder.binding as ItemMsgMyImgBinding).bean = list[position]
                (holder.binding as ItemMsgMyImgBinding).state = list[position].status
            }
            //发送语音
            MsgTypeStateEnum.MSG_TO_OTHER_VOICE -> {
                (holder.binding as ItemMsgMyAudioBinding).pos = position
                (holder.binding as ItemMsgMyAudioBinding).bean = list[position]
                (holder.binding as ItemMsgMyAudioBinding).state = list[position].status
            }
            //收到语音
            MsgTypeStateEnum.MSG_FROM_OTHER_VOICE -> {
                (holder.binding as ItemMsgOtherAudioBinding).pos = position
                (holder.binding as ItemMsgOtherAudioBinding).bean = list[position]
                (holder.binding as ItemMsgOtherAudioBinding).state = list[position].status
            }
            //发送文件
            MsgTypeStateEnum.MSG_TO_OTHER_FILE -> {
                (holder.binding as ItemMsgOtherFileBinding).pos = position
                (holder.binding as ItemMsgOtherFileBinding).bean = list[position]
                (holder.binding as ItemMsgOtherFileBinding).state = list[position].status
            }
            //收到文件
            MsgTypeStateEnum.MSG_FROM_OTHER_FILE -> {
                (holder.binding as ItemMsgMyFileBinding).pos = position
                (holder.binding as ItemMsgMyFileBinding).bean = list[position]
                (holder.binding as ItemMsgMyFileBinding).state = list[position].status
            }

            else -> {
                throw IllegalArgumentException("Invalid view type")
            }
        }


    }
}