package com.tenlionsoft.aimz_k.page.fragments

import android.app.AlertDialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.result.contract.ActivityResultContracts
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.FragmentContactBinding
import com.tenlionsoft.aimz_k.model.SelfContactBean
import com.tenlionsoft.aimz_k.page.activity.ApplyContactActivity
import com.tenlionsoft.aimz_k.page.activity.ChatActivity
import com.tenlionsoft.aimz_k.page.activity.MainActivity
import com.tenlionsoft.aimz_k.page.activity.SearchContactActivity
import com.tenlionsoft.aimz_k.viewmodel.ContactViewModel
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import com.tenlionsoft.baselib.widget.AdapterItemLongClickListener
import com.tenlionsoft.baselib.widget.LoadingDialog
import com.tenlionsoft.baselib.widget.StackedImageDecoration

class ContactFragment : Fragment(),
    AdapterItemLongClickListener<SelfContactBean>,
    AdapterItemClickListener<SelfContactBean> {

    companion object {
        fun newInstance() =
            ContactFragment()
    }

    private lateinit var viewModel: ContactViewModel
    private lateinit var mBind: FragmentContactBinding
    private lateinit var mActivity: MainActivity
    private var mLoading: LoadingDialog? = null

    private val launcher =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            val intent = it.data
            //判断是否要刷新fragment
        }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View {
        viewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return ContactViewModel(mActivity.applicationContext) as T
            }
        })[ContactViewModel::class.java]
        mBind = DataBindingUtil.inflate(
            layoutInflater, R.layout.fragment_contact, container, false
        )
        mBind.model = viewModel
        mBind.lifecycleOwner = this
        viewModel.itemClickListener = this
        viewModel.itemLongClickListener = this
        mBind.llTitle.llLoad.visibility = View.GONE
        mBind.llTitle.llSearchLayout.setOnClickListener {
            launcher.launch(
                Intent(
                    mActivity, SearchContactActivity::class.java
                )
            )
        }
        mBind.llNewContact.setOnClickListener {
            //判断是否存在好友申请
            if (viewModel.isHasApply.value!!) {
                launcher.launch(
                    Intent(
                        mActivity, ApplyContactActivity::class.java
                    )
                )
            }
        }
        viewModel.isHasApply.observe(viewLifecycleOwner) {
            if (it) {
                mBind.llNewContact.visibility = View.VISIBLE
            } else {
                mBind.llNewContact.visibility = View.GONE
            }
        }
        mBind.srlContent.setEnableLoadMore(false)
        mBind.srlContent.setOnRefreshListener {
            viewModel.doRefresh()
        }
        viewModel.isRefreshing.observe(viewLifecycleOwner) {
            if (!it) {
                mBind.srlContent.finishRefresh()
            }
        }
        viewModel.showLoadDialog.observe(viewLifecycleOwner) {
            if (it) {
                //显示loading
                mLoading =
                    LoadingDialog.Builder(mActivity)
                        .setCancelOutside(false)
                        .setCancelable(false)
                        .setMessage("删除中...")
                        .create()
                mLoading!!.show()
            } else {
                //隐藏loading
                mLoading?.dismiss()
                mLoading = null
            }
        }
        mBind.rlvContent.addItemDecoration(StackedImageDecoration())

        return mBind.root
    }

    override fun onResume() {
        super.onResume()
        viewModel.doGetContactApplyList()
    }


    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is MainActivity) {
            mActivity = context
        }
    }

    override fun onItemLongClick(
        type: Int, d: SelfContactBean
    ) {
        AlertDialog.Builder(mActivity).setTitle("警告")
            .setMessage("确定要删除该联系人吗？这样会删除与该联系人的所有聊天内容！")
            .setPositiveButton("确定") { dialog, _ ->
                dialog.dismiss()
                viewModel.delContact(d)
            }.setNegativeButton("取消") { dialog, _ -> dialog.dismiss() }.create().show()
    }

    override fun onItemClick(data: SelfContactBean) {
        mActivity.startActivity(
            Intent(mActivity, ChatActivity::class.java).putExtra("fromId", data.userId)
                .putExtra("name", data.userIdNickname)
        )
    }
}