package com.tenlionsoft.aimz_k.page.fragments

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.atwa.filepicker.core.FilePicker
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.FragmentMineBinding
import com.tenlionsoft.aimz_k.page.activity.MainActivity
import com.tenlionsoft.aimz_k.page.activity.ManageReplyActivity
import com.tenlionsoft.aimz_k.page.activity.UserInfoEditActivity
import com.tenlionsoft.aimz_k.viewmodel.MineViewModel
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.widget.LoadingDialog
import jp.wasabeef.glide.transformations.RoundedCornersTransformation

class MineFragment : Fragment() {
    private lateinit var mBind: FragmentMineBinding
    private lateinit var mActivity: MainActivity
    private val filePicker = FilePicker.getInstance(this)
    private var mLoading: LoadingDialog? = null
    private lateinit var viewModel: MineViewModel
    val requestOptions = RequestOptions.bitmapTransform(RoundedCornersTransformation(10, 0))
        .error(R.drawable.ic_user_default)
        .placeholder(R.drawable.ic_user_default)
        .skipMemoryCache(false)
        .diskCacheStrategy(DiskCacheStrategy.RESOURCE)

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBind = DataBindingUtil.inflate(layoutInflater, R.layout.fragment_mine, container, false)
        viewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return MineViewModel(mActivity) as T
            }
        })[MineViewModel::class.java]
        mBind.model = viewModel
        mBind.lifecycleOwner = this
        mBind.ivUserIcon.setOnClickListener {
            filePicker.pickImage { meta ->
                if (meta != null) {
                    viewModel.doUploadImg(meta)
                }
            }
        }
        initView()
        return mBind.root
    }

    private fun initView() {
        mBind.tvName.text = SpUtils.getNickName()
        mBind.tvAccount.text = SpUtils.getPhone()
        mBind.llSetting.setOnClickListener {
            startActivity(Intent(mActivity, ManageReplyActivity::class.java))
        }
        mBind.llExit.setOnClickListener {
            mActivity.reStartApp()
        }
        mBind.llUser.setOnClickListener {
            startActivity(Intent(mActivity, UserInfoEditActivity::class.java))
        }
        viewModel.showLoadDialog.observe(viewLifecycleOwner) {
            if (it) {
                //显示loading
                mLoading = LoadingDialog.Builder(mActivity)
                    .setCancelOutside(false)
                    .setCancelable(false)
                    .setMessage("上传中...")
                    .create()
                mLoading!!.show()
            } else {
                //隐藏loading
                mLoading?.dismiss()
                mLoading = null
            }
        }
        viewModel.isUploadIconSuccess.observe(viewLifecycleOwner) {
            if (it) {
                val icon = SpUtils.getAvatar().split(":")
                Glide.with(mActivity)
                    .load(NetConfig.FILE_DOWNLOAD_URL + icon[0])
                    .apply(requestOptions)
                    .into(mBind.ivUserIcon)
            }
        }
    }

    override fun onResume() {
        super.onResume()
        val icon = SpUtils.getAvatar().split(":")
        Glide.with(mActivity)
            .load(NetConfig.FILE_DOWNLOAD_URL + icon[0])
            .apply(requestOptions)
            .into(mBind.ivUserIcon)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is MainActivity) {
            mActivity = context
        }
    }


    companion object {
        fun newInstance() = MineFragment()
    }
}