package com.tenlionsoft.aimz_k.services

import android.annotation.SuppressLint
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.media.MediaPlayer
import android.os.Build
import android.os.IBinder
import android.util.Log
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.dao.DbManager
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.ReceiverBean
import com.tenlionsoft.aimz_k.model.SenderBean
import com.tenlionsoft.aimz_k.model.UserInfoBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.aimz_k.socket.WsManager
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.TimeUtils
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.disposables.Disposable
import io.reactivex.rxjava3.schedulers.Schedulers
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.util.concurrent.TimeUnit


/**
 * Socket service
 */
class SocketService : Service(), WsManager.MsgCallBack {
    private var mWsManager: WsManager? = null
    private val gson = Gson()
    private lateinit var mLocalReceiver: LocalReceiver
    private lateinit var mMediaPlayer: MediaPlayer
    private var msgObserver = arrayListOf<String>()
    private lateinit var mDispose: Disposable
    private val mainScope = MainScope()
    private val mGson = Gson()
    private val msgDao = DbManager.db.msgDao()
    private val cDao = DbManager.db.categoryDao()
    private val retrofitClient = RetrofitClient.getInstance(this)
    private val baseApi = retrofitClient.create(BaseApi::class.java)
    private val userMap = mutableMapOf<String, UserInfoBean>()
    override fun onCreate() {
        super.onCreate()
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val channel = NotificationChannel(
                ProjectConfig.NOTIFY_CHANNEL_ID,
                "Ai秒著客服",
                NotificationManager.IMPORTANCE_DEFAULT
            )
            val manager = getSystemService(NotificationManager::class.java)
            manager.createNotificationChannel(channel)
        }
    }


    @SuppressLint("ForegroundServiceType")
    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
//        val builder: Notification.Builder
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
//            builder = Notification.Builder(this, ProjectConfig.NOTIFY_CHANNEL_ID)
//        } else {
//            builder = Notification.Builder(this)
//        }
//        builder.setContentTitle("Ai秒著")
//            .setContentText("聊天已经登陆")
//            .setSmallIcon(R.drawable.app_logo_small)
//        val notification: Notification = builder.build()
//        startForeground(startId, notification)
        Log.e("SocketService", "onStartCommand:服务启动")
        this.startSocket() //开启Socket
        mMediaPlayer = MediaPlayer.create(this, com.tenlionsoft.baselib.R.raw.chat)
        registerLocalReceiver()
        return START_REDELIVER_INTENT
    }


    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    private fun registerLocalReceiver() {
        val intentFilter = IntentFilter()
        mLocalReceiver = LocalReceiver()
        intentFilter.addAction(ProjectConfig.A_S_MSG_SEND)//发送消息
        intentFilter.addAction(ProjectConfig.A_S_LOGOUT)  //断开socket
        intentFilter.addAction(ProjectConfig.A_S_RE_LOGIN)//重新连接socket
        registerReceiver(mLocalReceiver, intentFilter)
    }

    /**
     * 开启链接socket
     */
    private fun startSocket() {
        if (mWsManager == null) {
            mWsManager = WsManager.Builder(applicationContext)
                .wsUrl(NetConfig.WS_URL + "?token=" + SpUtils.getToken()).needReconnect(true)
                .addCallBack(this).build()
        }
        if (!mWsManager!!.isWsConnected()) {
            mWsManager!!.startConnect()
            subscribeMsgChange()
        }

    }

    private fun stopSocket() {
        if (mWsManager != null && mWsManager!!.isWsConnected()) {
            mWsManager!!.stopConnect()
            mWsManager = null
        }
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null
    }

    override fun onDestroy() {
        Log.e("SocketService", "onDestroy服务销毁")
        stopSocket()
        unregisterReceiver(mLocalReceiver)
        mMediaPlayer.release()
        if (!mDispose.isDisposed) {
            mDispose.dispose()
        }
        mainScope.cancel()
        super.onDestroy()
    }

    //发送消息
    inner class LocalReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
//            val isConnect = AppUtils?.isNetworkConnected(context)
//            if (true == isConnect) {
            when (intent?.action) {
                //发送消息
                ProjectConfig.A_S_MSG_SEND -> {
                    val msgConvertBean =
                        intent.getSerializableExtra(ProjectConfig.MSG_PUSH) as MsgConvertBean
                    sendMessage(msgConvertBean)
                }
                //断开连接
                ProjectConfig.A_S_LOGOUT -> {
                    stopSocket()
                    stopSelf()
                }
                //重新连接
                ProjectConfig.A_S_RE_LOGIN -> {
                    stopSocket()
                    startSocket()
                }
            }
//            } else {
//                ToastUtils.error("请检查网络")
//            }

        }

    }


    private fun subscribeMsgChange() {
        mDispose = Observable.interval(1500, TimeUnit.MILLISECONDS).subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread()).subscribe {
                if (msgObserver.isNotEmpty()) {
                    if (msgObserver.size > 1) {
                        if (!msgObserver.toString().contains("STATUS")) {
                            mMediaPlayer.start()
                        }
                        receiveMsgMulti(msgObserver)
                    } else {
                        if (!msgObserver[0].contains("STATUS")) {
                            mMediaPlayer.start()
                        }
                        receiveMsgSingle(msgObserver[0])
                    }
                    msgObserver.clear()
                }
            }
    }

    //发送消息
    private fun sendMessage(msgConvertBean: MsgConvertBean?) {
        if (msgConvertBean != null) {
            mainScope.launch {
                val msg = gson.toJson(msgConvertBean)
                Log.e("SocketService", "发送消息:${msg}")
                mWsManager?.sendMessage(msg)
                val msgBean = insertMsgBeanToDb(msgConvertBean)
//                val isExist = userMap.any { it.key == msgBean.receiverId }
//                if (isExist) {
//                    val userInfoBean = userMap[msgBean.receiverId]
//                    Log.e("SocketService", "已经存在${userInfoBean}")
//                    withContext(Dispatchers.IO) {
//                        //更新数据库
//                        cDao.updateInByMsgId(
//                            userInfoBean!!.nickname,
//                            userInfoBean.avatar,
//                            msgBean.messageId!!,
//                            SpUtils.getId()
//                        )
//                        msgDao.updateUserInfoByMessageId(
//                            sAvatar = SpUtils.getAvatar(),
//                            sNickName = SpUtils.getNickName(),
//                            rAvatar = userInfoBean.avatar,
//                            rNickName = userInfoBean.nickname,
//                            userInfoBean.nickname,
//                            userInfoBean.avatar,
//                            msgBean.messageId!!,
//                            SpUtils.getId()
//                        )
//                    }
//                } else {
//                    getUserInfo(
//                        msgBean.messageId!!, msgBean.receiverId!!, 23, ProjectConfig.MSG_SINGLE
//                    )
//                }
            }
        }
    }

    private fun sendRefresh() {
        Log.e("SocketService", "sendRefresh:通知页面刷新")
        val intent = Intent(ProjectConfig.A_S_UPDATE_INFO)
        sendBroadcast(intent)
    }

    //发送消息插入到数据库
    private suspend fun insertMsgBeanToDb(b: MsgConvertBean): MsgBean {
        return withContext(Dispatchers.IO) {
            val msgBean = ConvertBeanUtils.convertBeanToMsgBean(b)
            val uId = SpUtils.getId()
            msgDao.insertMsg(msgBean)
            cDao.updateOrInsert(
                uId, b.receiver.receiverId!!, b.sender.senderId!!, msgBean
            )
            return@withContext msgBean
        }
    }

    //收到单条消息
    private fun receiveMsgSingle(msgBean: String) {
        Log.e("SocketService", "单条信息:${msgBean}")
        mainScope.launch {
            val bean = ConvertBeanUtils.covertBean(msgBean)
            withContext(Dispatchers.IO) {
                when (bean) {
                    //状态
                    is CoverSealedBean.StateBean -> {
                        val statusType =
                            mGson.fromJson(bean.data.body, BodyContent::class.java).statusType!!
                        val mId = bean.data.messageId
                        val uId = SpUtils.getId()
                        val line = msgDao.updateStatus(mId, statusType, uId)
                        val id = cDao.updateStatByMessageId(
                            statusType, mId, uId
                        )
                        sendStatusBroadcast(bean.data.messageId, statusType)
                    }
                    //消息
                    is CoverSealedBean.Msg -> {
                        val uId = SpUtils.getId()
                        val rId = bean.msgBean.receiverId
                        val sId = bean.msgBean.senderId
                        val line = msgDao.insertMsg(bean.msgBean)
                        val l = cDao.updateOrInsert(
                            uId, rId!!, sId!!, bean.msgBean
                        )
                        Log.e(
                            "SocketService",
                            "receiveMsgSingle:登录人:$uId \n接收人:$rId\n发送人:$sId\n插入消息库:$line\n更新或者类型库:$l"
                        )
                        //发送接收成功消息
                        sendSuccessMsg(bean.msgBean)
                        //判断是否存在
//                        val isExist = userMap.any { it.key == bean.msgBean.receiverId }
//                        if (isExist) {
//                            val userInfoBean = userMap[bean.msgBean.receiverId]
//                            Log.e("SocketService", "已经存在${userInfoBean}")
//                            withContext(Dispatchers.IO) {
//                                //更新数据库
//                                cDao.updateInByMsgId(
//                                    userInfoBean!!.nickname,
//                                    userInfoBean.avatar,
//                                    bean.msgBean.messageId!!,
//                                    SpUtils.getId()
//                                )
//                                msgDao.updateUserInfoByMessageId(
//                                    sAvatar = userInfoBean.avatar,
//                                    sNickName = userInfoBean.nickname,
//                                    rAvatar = SpUtils.getAvatar(),
//                                    rNickName = SpUtils.getNickName(),
//                                    userInfoBean.nickname,
//                                    userInfoBean.avatar,
//                                    bean.msgBean.messageId!!,
//                                    SpUtils.getId()
//                                )
//                            }
//                        } else {
//                            getUserInfo(
//                                bean.msgBean.messageId!!,
//                                bean.msgBean.receiverId!!,
//                                23,
//                                ProjectConfig.MSG_SINGLE
//                            )
//                        }
                    }
                }
            }
            sendRefresh()
        }
    }


    //收到多条消息
    private fun receiveMsgMulti(msgList: ArrayList<String>) {
        Log.e("SocketService", "多条信息:${msgList}")
        msgList.forEach { str ->
            receiveMsgSingle(str)
        }
    }


    //发送消息状态
    private fun sendSuccessMsg(msgBean: MsgBean) {
        val successBean = buildStatusBean(msgBean.senderId!!, msgBean.messageId!!)
        mWsManager?.sendMessage(mGson.toJson(successBean))
    }

    //构建状态信息
    private fun buildStatusBean(receiverId: String, messageId: String): MsgConvertBean {
        val bodyBean = BodyContent(content = null, msg = "", statusType = "SUCCESS_RECEIVED")
        val body = mGson.toJson(bodyBean)
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = messageId,
            messageType = ProjectConfig.MSG_STATUS,
            metadata = "",
            timestamp = TimeUtils.getNowDateMillis(),
            sender = SenderBean(senderId = SpUtils.getId(), senderType = ""),
            receiver = ReceiverBean(receiverId = receiverId, receiverType = "SINGLE_USER"),
            status = "",
        )
    }

    //获取人员信息 23发送  24接收
    private fun getUserInfo(messageId: String, userId: String, status: Int, type: Int) {
        mainScope.launch {
            try {
                val userInfoBean = retrofitClient.makeApiCall {
                    baseApi.getUserInfo(userId)
                }
                if (userInfoBean != null) {
                    userMap[userId] = userInfoBean
                    Log.e("SocketService", "网络中获取:${userInfoBean}")
                    //更新数据库
//                    withContext(Dispatchers.IO) {
//                        //更新数据库
//                        cDao.updateInfoByMessageId(
//                            nickName = userInfoBean.nickname,
//                            avatar = userInfoBean.avatar,
//                            messageId = messageId,
//                            loginUserId = SpUtils.getId()
//                        )
//                        msgDao.updateUserInfoByMessageId(
//                            sAvatar = if (status == 23) SpUtils.getAvatar() else userInfoBean.avatar,
//                            sNickName = if (status == 23) SpUtils.getNickName() else userInfoBean.nickname,
//                            rAvatar = if (status == 23) userInfoBean.avatar else SpUtils.getAvatar(),
//                            rNickName = if (status == 23) userInfoBean.nickname else SpUtils.getNickName(),
//                            nickName = userInfoBean.nickname,
//                            avatar = userInfoBean.avatar,
//                            messageId = messageId,
//                            loginUserId = SpUtils.getId()
//                        )
//                    }
                }
                if (type == ProjectConfig.MSG_SINGLE) {
                    sendRefresh()
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    //发送更新状态广播
    private fun sendStatusBroadcast(messageId: String, status: String) {
        val intent =
            Intent(ProjectConfig.A_S_UPDATE_STATUS).putExtra(ProjectConfig.MSG_M_ID, messageId)
                .putExtra(ProjectConfig.MSG_M_STATUS, status)
        sendBroadcast(intent)
    }

    //socket接收到信息
    override fun onCallBackMsg(str: String) {
        msgObserver.add(str)
    }
}