package com.tenlionsoft.aimz_k.viewmodel

import android.content.Context
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.tenlionsoft.aimz_k.adapter.ApplyContactUserIconAdapter
import com.tenlionsoft.aimz_k.adapter.ContactAdapter
import com.tenlionsoft.aimz_k.dao.DbManager
import com.tenlionsoft.aimz_k.model.ApplyContactBean
import com.tenlionsoft.aimz_k.model.SelfContactBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.net.ExParse
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import com.tenlionsoft.baselib.widget.AdapterItemLongClickListener
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class ContactViewModel(private val context: Context) : BaseViewModel() {
    private val _contactList = MutableLiveData<List<SelfContactBean>?>()
    val adapter = ContactAdapter(
        _contactList.value
            ?: emptyList(), this
    )
    private val _applyList = MutableLiveData<List<ApplyContactBean>>()
    val applyAdapter = ApplyContactUserIconAdapter(
        _applyList.value
            ?: emptyList()
    )
    var itemClickListener: AdapterItemClickListener<SelfContactBean>? = null
    var itemLongClickListener: AdapterItemLongClickListener<SelfContactBean>? = null
    val isHasApply = MutableLiveData<Boolean>(false)
    val isRefreshing = MutableLiveData<Boolean>(false)
    private val retrofit = RetrofitClient.getInstance(context)
    private val userApi = retrofit.create(BaseApi::class.java)

    init {
        getContactList()
    }

    //获取联系人列表
    private fun getContactList() {
        viewModelScope.launch {
            try {
                val pageBean = retrofit.makeApiCall {
                    userApi.getMineContactList()
                }
                if (pageBean.isNotEmpty()) {
                    _contactList.value = pageBean
                } else {
                    _contactList.value = emptyList()
                }
                adapter.setData(_contactList.value!!)
                isRefreshing.value = false
            } catch (e: Exception) {
                e.printStackTrace()
                isRefreshing.value = false
                ExParse.parse(e)
            }
        }
    }

    fun doRefresh() {
        isRefreshing.value = true
        _contactList.value = emptyList()
        adapter.setData(_contactList.value!!)
        getContactList()
    }


    //获取申请人列表
    fun doGetContactApplyList() {
        viewModelScope.launch {
            try {
                val pageBean = retrofit.makeApiCall {
                    userApi.doGetMyApplyContactList()
                }
                if (pageBean.isNotEmpty()) {
                    pageBean.removeAll(pageBean.filter { it.status == "PASS" })
                    if (pageBean.size > 0) {
                        isHasApply.value = true
                        _applyList.value = pageBean
                    } else {
                        isHasApply.value = false
                        _applyList.value = emptyList()
                    }
                } else {
                    isHasApply.value = false
                    _applyList.value = emptyList()
                }
                applyAdapter.setData(_applyList.value!!)
            } catch (e: Exception) {
                e.printStackTrace()
                ExParse.parse(e)
            }
        }
    }

    //删除联系人
    fun delContact(d: SelfContactBean) {
        viewModelScope.launch {
            showLoadDialog.value = true
            try {
                val responseBean = retrofit.makeApiCall {
                    userApi.doDelContact(d.contactUserId)
                }
                withContext(Dispatchers.IO) {
                    val cDao = DbManager.db.categoryDao()
                    val mDao = DbManager.db.msgDao()
                    cDao.delChatBySenderId(SpUtils.getId(), d.userId, SpUtils.getId())
                    mDao.delChatHistory(SpUtils.getId(), SpUtils.getId(), d.userId)
                }
                if (responseBean.code == 200) {
                    ToastUtils.success("删除成功")
                    doRefresh()
                } else {
                    ToastUtils.error(
                        responseBean.msg
                            ?: "删除失败,请稍后重试"
                    )
                }
                showLoadDialog.value = false
            } catch (e: Exception) {
                e.printStackTrace()
                ExParse.parse(e)
                showLoadDialog.value = false
            }

        }
    }


}