package com.tenlionsoft.aimz_k.viewmodel

import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.model.AppTokenUser
import com.tenlionsoft.aimz_k.model.LoginUser
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.App
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.net.ExParse
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.JwtUtils
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import kotlinx.coroutines.launch
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.RequestBody.Companion.toRequestBody


class LoginPageViewModel : BaseViewModel() {
    var userName: String = "zs"
    var userPwd: String = "123456"
    val isLoginSuccess = MutableLiveData<Boolean>()
    private val gson = Gson()
    private val retrofitClient = RetrofitClient.getInstance(App.context)
    private val userApi = retrofitClient.create(BaseApi::class.java)



    fun onUserNameChange(s: CharSequence, start: Int, before: Int, count: Int) {
        this.userName = s.toString();
    }

    fun onUserPwdChange(s: CharSequence, start: Int, before: Int, count: Int) {
        this.userPwd = s.toString();
    }

    fun doLogin() {
//        isLoginSuccess.value= true
        val isLegal = checkParams();
        if (isLegal) {
            viewModelScope.launch {
                showLoadDialog.value = true;//显示加载框
                login();
            }
        }
    }

    /**
     * 登陆
     */
    private suspend fun login() {
        try {
            val userBody = LoginUser(
                userPwd,
                userName
            )

            val loginUserStr = gson.toJson(userBody)
            val body =
                loginUserStr.toRequestBody("application/json; charset=utf-8".toMediaTypeOrNull())

            val user = retrofitClient.makeApiCall {
                userApi.doLogin(body)
            }
            if (user.code == 200) {
                //登陆成功
                val token = JwtUtils.parseToken(user.data)
                if (token.isEmpty()) {
                    ToastUtils.error("登陆失败,请稍后重试")
                    loginFailed()
                } else {
                    //登陆成功
                    //解析Token
                    val appTokenUser = gson.fromJson(token, AppTokenUser::class.java)
                    Log.e("LoginPageViewModel", "login:${appTokenUser.user}")
                    SpUtils.putPassword(userPwd)
                    SpUtils.putAvatar(appTokenUser.user.avatar)
                    SpUtils.putNickName(appTokenUser.user.nickname)
                    SpUtils.putId(appTokenUser.user.userId)
                    SpUtils.putEmail(appTokenUser.user.email)
                    SpUtils.putUserName(appTokenUser.user.username)
                    SpUtils.putPhone(appTokenUser.user.phone)
                    SpUtils.putToken(user.data)
                    showLoadDialog.value = false
                    isLoginSuccess.value = true
                }
            } else {
                //登陆失败
                ToastUtils.error(
                    user.msg
                        ?: "登录失败,请稍后重试"
                )
                loginFailed()
            }
        } catch (e: Exception) {
            e.printStackTrace()
            loginFailed()
            ExParse.parse(e)
        }
    }

    private fun checkParams(): Boolean {
        if (userName.isEmpty()) {
            ToastUtils.error("请输入用户名")
            return false;
        }
        if (userPwd.isEmpty()) {
            ToastUtils.error("请输入密码")
            return false;
        }
        return true;
    }

    private fun loginFailed() {
        showLoadDialog.value = false
        isLoginSuccess.value = false
    }
}