package com.tenlionsoft.baselib.utils

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import com.tenlionsoft.baselib.model.ImageSize
import java.io.ByteArrayOutputStream
import java.io.IOException

object ImageUtils {
    fun getImageSize(bitmap: Bitmap?): ImageSize? {
        val imageSize: ImageSize = ImageSize()
        if (null == bitmap || bitmap.isRecycled) {
            return null
        }
        val baos = ByteArrayOutputStream()
        bitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos)
        val byteTmp = baos.toByteArray()
        try {
            baos.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
        val bitmapOptions = BitmapFactory.Options()
        bitmapOptions.inJustDecodeBounds = true
        BitmapFactory.decodeByteArray(byteTmp, 0, byteTmp.size, bitmapOptions)
        val outWidth = bitmapOptions.outWidth
        val outHeight = bitmapOptions.outHeight
        val maxWidth = 400
        val maxHeight = 400
        val minWidth = 150
        val minHeight = 150
        if (outWidth / maxWidth > outHeight / maxHeight) { //
            if (outWidth >= maxWidth) { //
                imageSize.setWidth(maxWidth)
                imageSize.setHeight(outHeight * maxWidth / outWidth)
            } else {
                imageSize.setWidth(outWidth)
                imageSize.setHeight(outHeight)
            }
            if (outHeight < minHeight) {
                imageSize.setHeight(minHeight)
                val width = outWidth * minHeight / outHeight
                imageSize.setWidth(Math.min(width, maxWidth))
            }
        } else {
            if (outHeight >= maxHeight) {
                imageSize.setHeight(maxHeight)
                imageSize.setWidth(outWidth * maxHeight / outHeight)
            } else {
                imageSize.setHeight(outHeight)
                imageSize.setWidth(outWidth)
            }
            if (outWidth < minWidth) {
                imageSize.setWidth(minWidth)
                val height = outHeight * minWidth / outWidth
                imageSize.setHeight(Math.min(height, maxHeight))
            }
        }

        return imageSize
    }
}