package com.tenlionsoft.baselib.widget

import android.app.Dialog
import android.content.Context
import android.graphics.Rect
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.util.DisplayMetrics
import android.view.Gravity
import android.view.LayoutInflater
import android.view.TouchDelegate
import android.view.View
import android.view.WindowManager
import android.widget.ImageView
import com.tenlionsoft.baselib.R

class CenterProgressUpdateView private constructor(
    private val context: Context,
    private val content: String
) :
    Dialog(
        context,
        R.style.dialog_center
    ) {
    private var view: View? = null
    var mIvImg: ImageView? = null
    private var mCpProgress: CircleProgress? = null




    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        view = LayoutInflater.from(context).inflate(R.layout.dialog_center_update_progress, null)
        mCpProgress = view!!.findViewById(R.id.cp_progress)
        setContentView(view!!)
        setCancelable(false)
        setCanceledOnTouchOutside(false)
        val dm = DisplayMetrics()
        val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        windowManager.defaultDisplay.getMetrics(dm)
        val window = this.window
        window!!.setGravity(Gravity.CENTER)
        window.setWindowAnimations(R.style.Dialog_ChoseFile)
        val params = window.attributes
        params.width = (dm.widthPixels * 0.7).toInt()
        params.height = WindowManager.LayoutParams.WRAP_CONTENT
        window.attributes = params
        window.setBackgroundDrawable(ColorDrawable())
    }


    /**
     * 将view的点击区域放大
     *
     * @param view             需要放大的view
     * @param expandTouchWidth 放大的宽度
     */
    fun setTouchDelegate(view: View, expandTouchWidth: Int) {
        val parentView = view.parent as View
        parentView.post {
            val rect = Rect()
            view.getHitRect(rect)
            rect.top -= expandTouchWidth
            rect.bottom += expandTouchWidth
            rect.left -= expandTouchWidth
            rect.right += expandTouchWidth
            val touchDelegate = TouchDelegate(rect, view)
            parentView.touchDelegate = touchDelegate
        }
    }
    /**
     * 设置当前进度
     */
    fun setCurrentProgress(progress: Int) {
        mCpProgress?.setCurrentProgress(progress.toFloat())
    }


    class DialogBuilder(private val mContext: Context) {
        private var content: String? = null

        fun build(): CenterProgressUpdateView {
            return CenterProgressUpdateView(this.mContext, content!!)
        }

        fun setContent(content: String?): DialogBuilder {
            this.content = content
            return this
        }
    }
}