package com.tenlionsoft.medialib.base

import android.view.View
import android.view.WindowManager
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import com.shuyu.gsyvideoplayer.GSYVideoManager
import com.shuyu.gsyvideoplayer.utils.OrientationUtils
import com.tenlionsoft.baselib.base.BaseActivity
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.medialib.databinding.ActivitySimpleVideoBinding


class SimpleVideoActivity : BaseActivity() {
    private lateinit var mBind: ActivitySimpleVideoBinding
    private lateinit var orientationUtils: OrientationUtils
    override fun bindView() {
        mBind = DataBindingUtil.setContentView(
            this@SimpleVideoActivity, com.tenlionsoft.medialib.R.layout.activity_simple_video
        )
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        window.statusBarColor = ContextCompat.getColor(this, com.tenlionsoft.baselib.R.color.black)
        val source = intent.getStringExtra("url")
        val title = intent.getStringExtra("title") ?: "视频预览"
        if (source.isNullOrEmpty()) {
            ToastUtils.error("视频地址有误")
            finish()
            return
        }
        mBind.sVideo.setUp(NetConfig.MAIN_URL + source, true, title)


        //增加封面
//        val imageView = ImageView(this)
//        imageView.scaleType = ImageView.ScaleType.CENTER_CROP
//        imageView.setImageResource(com.tenlionsoft.medialib.R.mipmap.xxx1)
//        videoPlayer.setThumbImageView(imageView)

        //增加title
        mBind.sVideo.titleTextView.visibility = View.VISIBLE

        //设置返回键
        mBind.sVideo.backButton.visibility = View.VISIBLE

        //设置旋转
        orientationUtils = OrientationUtils(this, mBind.sVideo)

        //设置全屏按键功能,这是使用的是选择屏幕，而不是全屏
        mBind.sVideo.getFullscreenButton().setOnClickListener(View.OnClickListener {
            // ------- ！！！如果不需要旋转屏幕，可以不调用！！！-------
            // 不需要屏幕旋转，还需要设置 setNeedOrientationUtils(false)
            orientationUtils.resolveByClick()
            //finish();
        })

        //是否可以滑动调整
        mBind.sVideo.setIsTouchWiget(true)

        //设置返回按键功能
        mBind.sVideo.getBackButton().setOnClickListener(View.OnClickListener { onBackPressed() })


        /**不需要屏幕旋转 */
        mBind.sVideo.setNeedOrientationUtils(false)

        mBind.sVideo.startPlayLogic()
    }

    override fun onResume() {
        super.onResume()
        mBind.sVideo.onVideoResume()
    }

    override fun onPause() {
        super.onPause()
        mBind.sVideo.onVideoPause()
    }

    override fun onBackPressed() {
        /**       不需要回归竖屏 */
//        if (orientationUtils.getScreenType() == ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE) {
//            videoPlayer.getFullscreenButton().performClick();
//            return;
//        }
        //释放所有
        mBind.sVideo.setVideoAllCallBack(null)
        super.onBackPressed()
    }

    override fun onDestroy() {
        super.onDestroy()
        GSYVideoManager.releaseAllVideos()
        orientationUtils.releaseListener()
    }
}